@@ -1,54 +1,37 @@
-import json, django
-from future.builtins import str, int
+import json
+from distutils.version import StrictVersion
+
 from django.conf import settings
-from django.core.urlresolvers import reverse
+from django.conf.urls import url
+from django.contrib.auth import get_permission_codename
+from django.contrib.admin import SimpleListFilter
+from django.core.exceptions import ValidationError
 from django.db import transaction
-from django.http import HttpResponseNotFound, HttpResponse, HttpResponseBadRequest, HttpResponseRedirect
-from django.utils.translation import ugettext_lazy as _
-from polymorphic.admin import PolymorphicParentModelAdmin, PolymorphicModelChoiceForm
-from polymorphic_tree.models import PolymorphicMPTTModel
+from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotFound, HttpResponseRedirect
+from django.urls import reverse
+from django.utils.translation import gettext_lazy as _, gettext
 from mptt.admin import MPTTModelAdmin
-
-try:
-    # Django 1.6 requires this
-    from django.conf.urls import url
-except ImportError:
-    # Django 1.3 compatibility
-    from django.conf.urls.defaults import url
-
-
-try:
-    transaction_atomic = transaction.atomic
-except AttributeError:
-    transaction_atomic = transaction.commit_on_success
+from mptt.exceptions import InvalidMove
+from polymorphic.admin import PolymorphicModelChoiceForm, PolymorphicParentModelAdmin
+from polymorphic_tree.models import PolymorphicMPTTModel
 
 
 class NodeTypeChoiceForm(PolymorphicModelChoiceForm):
     type_label = _("Node type")
 
 
-try:
-    from django.contrib.admin import SimpleListFilter
-except ImportError:
-    extra_list_filters = ()
-else:
-    # Django 1.4+:
-    class NodeTypeListFilter(SimpleListFilter):
-        parameter_name = 'ct_id'
-        title = _('node type')
+class NodeTypeListFilter(SimpleListFilter):
+    parameter_name = 'ct_id'
+    title = _('node type')
 
-        def lookups(self, request, model_admin):
-            if django.VERSION >= (1, 6):
-                return model_admin.get_child_type_choices(request, 'change')
-            return model_admin.get_child_type_choices()
+    def lookups(self, request, model_admin):
+        return model_admin.get_child_type_choices(request, 'change')
 
-        # Whoops: Django 1.6 didn't rename this one!
-        def queryset(self, request, queryset):
-            if self.value():
-                queryset = queryset.filter(polymorphic_ctype_id=self.value())
-            return queryset
-
-    extra_list_filters = (NodeTypeListFilter,)
+    # Whoops: Django 1.6 didn't rename this one!
+    def queryset(self, request, queryset):
+        if self.value():
+            queryset = queryset.filter(polymorphic_ctype_id=self.value())
+        return queryset
 
 
 class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmin):
@@ -62,7 +45,7 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
     add_type_form = NodeTypeChoiceForm
 
     # Config list page:
-    list_filter = extra_list_filters
+    list_filter = (NodeTypeListFilter,)
 
     # TODO: disable the pagination in the admin, because it doesn't work with the current template code.
     # This is a workaround for https://github.com/edoburu/django-polymorphic-tree/issues/2 until
@@ -79,12 +62,10 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         templates.insert(-1, 'admin/polymorphic_tree/change_list.html')  # Just before admin/change_list.html
         return templates
 
-
     # NOTE: the regular results table is replaced client-side with a jqTree list.
     # When making changes to the list, test both the JavaScript and non-JavaScript variant.
     # The jqTree variant still uses the server-side rendering for the columns.
 
-
     def actions_column(self, node):
         """
         An extra column to display action icons.
@@ -95,15 +76,15 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
     actions_column.allow_tags = True
     actions_column.short_description = _('Actions')
 
-
     def get_action_icons(self, node):
         """
         Return a list of all action icons in the :func:`actions_column`.
         """
         actions = []
-        if self.can_have_children(node):
+        if node.can_have_children:
             actions.append(
-                u'<a href="add/?{parent_attr}={id}" title="{title}" class="add-child-object"><img src="{static}polymorphic_tree/icons/page_new.gif" width="16" height="16" alt="{title}" /></a>'.format(
+                u'<a href="add/?{parent_attr}={id}" title="{title}" class="add-child-object">'
+                u'<img src="{static}polymorphic_tree/icons/page_new.gif" width="16" height="16" alt="{title}" /></a>'.format(
                     parent_attr=self.model._mptt_meta.parent_attr, id=node.pk, title=_('Add sub node'), static=settings.STATIC_URL)
             )
         else:
@@ -111,7 +92,8 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
 
         if self.can_preview_object(node):
             actions.append(
-                u'<a href="{url}" title="{title}" target="_blank"><img src="{static}polymorphic_tree/icons/world.gif" width="16" height="16" alt="{title}" /></a>'.format(
+                u'<a href="{url}" title="{title}" target="_blank">'
+                u'<img src="{static}polymorphic_tree/icons/world.gif" width="16" height="16" alt="{title}" /></a>'.format(
                     url=node.get_absolute_url(), title=_('View on site'), static=settings.STATIC_URL)
             )
 
@@ -121,28 +103,12 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         actions.append(u'<span class="no-js">{0}{1}</span>'.format(move_up, move_down))
         return actions
 
-
     def can_preview_object(self, node):
         """
         Define whether a node can be previewed.
         """
         return hasattr(node, 'get_absolute_url')
 
-
-    def can_have_children(self, node):
-        """
-        Define whether a node can have children.
-        """
-        # Allow can_have_children to be either to be a property on the base class that always works.
-        if not node.can_have_children:
-            return False
-
-        # or a static variable declared on the class (avoids need for downcasted models).
-        NodeClass = node.get_real_instance_class()
-        return bool(NodeClass.can_have_children)
-
-
-
     # ---- Custom views ----
 
     def get_urls(self):
@@ -158,24 +124,26 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         ]
         return extra_urls + base_urls
 
-
     @property
     def api_node_moved_view_url(self):
         # Provided for result list template
         info = _get_opt(self.model)
-        return reverse('admin:{0}_{1}_moved'.format(*info))
+        return reverse('admin:{0}_{1}_moved'.format(*info), current_app=self.admin_site.name)
 
-
-    @transaction_atomic
+    @transaction.atomic
     def api_node_moved_view(self, request):
         """
         Update the position of a node, from a API request.
         """
         try:
-            moved_id = int(request.POST['moved_id'])
-            target_id = int(request.POST['target_id'])
+            moved_id = _get_pk_value(request.POST['moved_id'])
+            target_id = _get_pk_value(request.POST['target_id'])
             position = request.POST['position']
-            previous_parent_id = int(request.POST['previous_parent_id']) or None
+
+            if request.POST.get('previous_parent_id'):
+                previous_parent_id = _get_pk_value(request.POST['previous_parent_id'])
+            else:
+                previous_parent_id = None
 
             # Not using .non_polymorphic() so all models are downcasted to the derived model.
             # This causes the signal below to be emitted from the proper class as well.
@@ -186,26 +154,40 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
         except self.model.DoesNotExist as e:
             return HttpResponseNotFound(json.dumps({'action': 'reload', 'error': str(e[0])}), content_type='application/json')
 
-        if not self.can_have_children(target) and position == 'inside':
+        if not request.user.has_perm(get_permission_codename('change', moved._meta), moved):
             return HttpResponse(json.dumps({
                 'action': 'reject',
                 'moved_id': moved_id,
-                'error': _(u'Cannot place \u2018{0}\u2019 below \u2018{1}\u2019; a {2} does not allow children!').format(moved, target, target._meta.verbose_name)
-            }), content_type='application/json', status=409)  # Conflict
-        if moved.parent_id != previous_parent_id:
+                'error': gettext('You do not have permission to move this node.')
+            }), content_type='application/json', status=409)
+
+        # Compare on strings to support UUID fields.
+        parent_attr_id = '{}_id'.format(moved._mptt_meta.parent_attr)
+        if str(getattr(moved, parent_attr_id)) != str(previous_parent_id):
             return HttpResponse(json.dumps({
                 'action': 'reload',
                 'error': 'Client seems to be out-of-sync, please reload!'
             }), content_type='application/json', status=409)
 
-        # TODO: with granular user permissions, check if user is allowed to edit both pages.
-
         mptt_position = {
             'inside': 'first-child',
             'before': 'left',
             'after': 'right',
         }[position]
-        moved.move_to(target, mptt_position)
+        try:
+            moved.move_to(target, mptt_position)
+        except ValidationError as e:
+            return HttpResponse(json.dumps({
+                'action': 'reject',
+                'moved_id': moved_id,
+                'error': '\n'.join(e.messages)
+            }), content_type='application/json', status=409)  # Conflict
+        except InvalidMove as e:
+            return HttpResponse(json.dumps({
+                'action': 'reject',
+                'moved_id': moved_id,
+                'error': str(e)
+            }), content_type='application/json', status=409)
 
         # Some packages depend on calling .save() or post_save signal after updating a model.
         # This is required by django-fluent-pages for example to update the URL caches.
@@ -219,7 +201,6 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
             'action_column': self.actions_column(moved),
         }), content_type='application/json')
 
-
     def move_up_view(self, request, object_id):
         node = self.model.objects.get(pk=object_id)
 
@@ -230,7 +211,6 @@ class PolymorphicMPTTParentModelAdmin(PolymorphicParentModelAdmin, MPTTModelAdmi
 
         return HttpResponseRedirect('../../')
 
-
     def move_down_view(self, request, object_id):
         node = self.model.objects.get(pk=object_id)
 
@@ -248,3 +228,9 @@ def _get_opt(model):
     except AttributeError:
         return model._meta.app_label, model._meta.module_name
 
+
+def _get_pk_value(text):
+    try:
+        return int(text)
+    except ValueError:
+        return text  # Allow uuid fields
