@@ -5,7 +5,6 @@ from django.utils.safestring import mark_safe
 from mptt.templatetags.mptt_tags import cache_tree_children
 from polymorphic_tree.templatetags.stylable_admin_list import stylable_column_repr
 
-
 register = Library()
 
 
@@ -28,6 +27,7 @@ def mptt_breadcrumb(node):
 
 
 class AdminListRecurseTreeNode(Node):
+
     def __init__(self, template_nodes, cl_var):
         self.template_nodes = template_nodes
         self.cl_var = cl_var
@@ -53,7 +53,7 @@ class AdminListRecurseTreeNode(Node):
             bits.append(self._render_node(context, cl, child))
 
         columns = self._get_column_repr(cl, node)  # list(tuple(name, html), ..)
-        first_real_column = (col for col in columns if col[0] != 'action_checkbox').next()
+        first_real_column = next(col for col in columns if col[0] != 'action_checkbox')
 
         context['columns'] = columns
         context['other_columns'] = [col for col in columns if col[0] not in ('action_checkbox', first_real_column[0])]
