@@ -11,20 +11,18 @@ to the class, to define custom classes for a column.
 
 This feature can be activated by simply extending the template stylable/admin/change_list.html
 """
-from future.builtins import zip, str
+import django
 from django.conf import settings
-from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
-from django.core.exceptions import ObjectDoesNotExist
-from django.db import models
-from django.utils import formats
-from django.utils.html import escape, conditional_escape
-from django.utils.safestring import mark_safe
-from django.utils.encoding import smart_unicode, force_unicode
-from django.template import Library
 from django.contrib.admin.templatetags.admin_list import _boolean_icon, result_headers
+from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist
+from django.db import models
+from django.template import Library
+from django.utils import formats, timezone
+from django.utils.encoding import force_str, smart_str
+from django.utils.html import conditional_escape, escape
+from django.utils.safestring import SafeData, mark_safe
 from tag_parser.basetags import BaseInclusionNode
 
-
 # While this is based on mptt/templatetags/mptt_admin.py,
 # and django/contrib/admin/templatetags/admin_list.py,
 # much has been changed, simplified, and refactored for clarity.
@@ -47,10 +45,20 @@ class StylableResultList(BaseInclusionNode):
 
     def get_context_data(self, parent_context, *tag_args, **tag_kwargs):
         cl = tag_args[0]
+
+        if 'grappelli' in settings.INSTALLED_APPS:
+            theme_css = 'polymorphic_tree/adminlist/nodetree_grappelli.css'
+        elif 'classic_theme' in settings.INSTALLED_APPS:
+            theme_css = 'polymorphic_tree/adminlist/nodetree_classic.css'
+        else:
+            # flat theme aka Django 1.9 default
+            theme_css = 'polymorphic_tree/adminlist/nodetree_flat.css'
+
         return {
             'cl': cl,
             'result_headers': list(stylable_result_headers(cl)),
             'results': list(stylable_results(cl)),
+            'nodetree_theme_css': theme_css,
 
             # added for frontend
             'has_add_permission': parent_context['has_add_permission'],
@@ -74,7 +82,6 @@ def stylable_result_headers(cl):
     for field_name, header in zip(cl.list_display, result_headers(cl)):
         header['field_name'] = field_name  # For JavaScript
 
-
         if header.get('class_attrib'):
             # Remove any sorting marker for mptt tables, because they are not sortable.
             if hasattr(cl.model, '_mptt_meta'):
@@ -91,6 +98,7 @@ def stylable_result_headers(cl):
 
 
 class ResultListRow(list):
+
     def __init__(self, seq, object):
         super(ResultListRow, self).__init__(seq)
         self.object = object
@@ -129,7 +137,7 @@ def stylable_items_for_result(cl, result, form):
 
         # This is all standard stuff, refactored to separate methods.
         result_repr, row_classes = stylable_column_repr(cl, result, field_name)
-        if force_unicode(result_repr) == '':
+        if force_str(result_repr) == '':
             result_repr = mark_safe('&nbsp;')
 
         # Custom stuff, select row classes
@@ -159,25 +167,25 @@ def stylable_items_for_result(cl, result, form):
                 else:
                     attr = pk
                 value = result.serializable_value(attr)
-                result_id = repr(force_unicode(value))[1:]
+                result_id = repr(force_str(value))[1:]
                 link_attr += ' onclick="opener.dismissRelatedLookupPopup(window, %s); return false;"' % result_id
 
-            yield mark_safe(u'<%s%s><a href="%s"%s>%s</a></%s>' % \
-                (table_tag, row_attr, url, link_attr, conditional_escape(result_repr), table_tag))
+            yield mark_safe(u'<%s%s><a href="%s"%s>%s</a></%s>' %
+                            (table_tag, row_attr, url, link_attr, conditional_escape(result_repr), table_tag))
         else:
             # By default the fields come from ModelAdmin.list_editable,
             # but if we pull the fields out of the form instead,
             # custom ModelAdmin instances can provide fields on a per request basis
             if form and field_name in form.fields:
                 bf = form[field_name]
-                result_repr = mark_safe(force_unicode(bf.errors) + force_unicode(bf))
+                result_repr = mark_safe(force_str(bf.errors) + force_str(bf))
             else:
                 result_repr = conditional_escape(result_repr)
 
             yield mark_safe(u'<td%s>%s</td>' % (row_attr, result_repr))
 
     if form:
-        yield mark_safe(u'<td>%s</td>' % force_unicode(form[cl.model._meta.pk.name]))
+        yield mark_safe(u'<td>%s</td>' % force_str(form[cl.model._meta.pk.name]))
 
 
 def _get_mptt_indent_field(cl, result):
@@ -192,8 +200,8 @@ def _get_mptt_indent_field(cl, result):
     mptt_indent_field = None
     for field_name in cl.list_display:
         try:
-            f = cl.lookup_opts.get_field(field_name)
-        except models.FieldDoesNotExist:
+            cl.lookup_opts.get_field(field_name)
+        except FieldDoesNotExist:
             if mptt_indent_field is None:
                 attr = getattr(result, field_name, None)
                 if callable(attr):
@@ -213,11 +221,11 @@ def stylable_column_repr(cl, result, field_name):
     """
     try:
         f = cl.lookup_opts.get_field(field_name)
-    except models.FieldDoesNotExist:
+    except FieldDoesNotExist:
         return _get_non_field_repr(cl, result, field_name)  # Field not found (maybe a function)
     else:
         row_classes = None
-        value = display_for_field(getattr(result, f.attname), f)  # Standard field
+        value = display_for_field(getattr(result, f.attname), f, cl.model_admin.get_empty_value_display())  # Standard field
         if isinstance(f, models.DateField) or isinstance(f, models.TimeField):
             row_classes = ['nowrap']
         return value, row_classes
@@ -236,7 +244,7 @@ def _get_non_field_repr(cl, result, field_name):
         if callable(field_name):
             attr = field_name
             value = attr(result)
-        elif hasattr(cl.model_admin, field_name) and not field_name in ('__str__', '__unicode__'):
+        elif hasattr(cl.model_admin, field_name) and field_name not in ('__str__', '__unicode__'):
             attr = getattr(cl.model_admin, field_name)
             value = attr(result)
         else:
@@ -252,11 +260,14 @@ def _get_non_field_repr(cl, result, field_name):
         if boolean:
             allow_tags = True
             result_repr = _boolean_icon(value)
+        elif isinstance(value, SafeData):
+            allow_tags = True
+            result_repr = value
         else:
-            result_repr = smart_unicode(value)
+            result_repr = smart_str(value)
 
     except (AttributeError, ObjectDoesNotExist):
-        result_repr = EMPTY_CHANGELIST_VALUE
+        result_repr = cl.model_admin.get_empty_value_display()
     else:
         # Strip HTML tags in the resulting text, except if the
         # function has an "allow_tags" attribute set to True.
@@ -269,23 +280,24 @@ def _get_non_field_repr(cl, result, field_name):
 
 
 # from Django 1.4:
-def display_for_field(value, field):
+def display_for_field(value, field, empty_value_display):
     from django.contrib.admin.templatetags.admin_list import _boolean_icon
-    from django.contrib.admin.views.main import EMPTY_CHANGELIST_VALUE
 
     if field.flatchoices:
-        return dict(field.flatchoices).get(value, EMPTY_CHANGELIST_VALUE)
+        return dict(field.flatchoices).get(value, empty_value_display)
     # NullBooleanField needs special-case null-handling, so it comes
     # before the general null test.
     elif isinstance(field, models.BooleanField) or isinstance(field, models.NullBooleanField):
         return _boolean_icon(value)
     elif value is None:
-        return EMPTY_CHANGELIST_VALUE
+        return empty_value_display
     elif isinstance(field, models.DateField) or isinstance(field, models.TimeField):
+        if isinstance(field, models.DateTimeField):
+            value = timezone.localtime(value)
         return formats.localize(value)
     elif isinstance(field, models.DecimalField):
         return formats.number_format(value, field.decimal_places)
     elif isinstance(field, models.FloatField):
         return formats.number_format(value)
     else:
-        return smart_unicode(value)
+        return smart_str(value)
